-- Copyright 2024 by Todd Hundersmarck (ThundR)
-- All Rights Reserved


THMain = {}
local THMain_mt = THUtils.createClass(THMain, THCore)
local THMain_dataKey = "thCameraSettings"
THMain.MODULE = {}
function THMain.new(xmlDataKey, customMt)
    xmlDataKey = xmlDataKey or THMain_dataKey
    customMt = THUtils.getNoNil(customMt, THMain_mt)
    local self = THCore.new(xmlDataKey, customMt)
    return self
end
function THMain:onInitialize()
    if not self.isInitFinished then
        self.xmlSchema = THMain.xmlSchema
        self.xmlSchemaSettings = THMain.xmlSchemaSettings
        self.xmlSchemaSavegame = THMain.xmlSchemaSavegame
        THMain:superClass().onInitialize(self)
        self:initSettings()
        self.isInitFinished = true
    end
end
function THMain:initSettings()
    local settingsManager = self.settingsManager
    local categoryMain = settingsManager:addSettingCategory("main", "$l10n_thTitle_constructionCamera")
    local settingConstructionMaxZoom = settingsManager:addSetting("constructionCameraMaxZoomFactor", categoryMain, "$l10n_thTitle_constructionCameraMaxZoomFactor", "$l10n_thToolTip_constructionCameraMaxZoomFactor", THSettingsManager.ELEMENT_TYPE.MULTI_OPTION)
    for zoomPct = 100, 500, 25 do
        local zoomFactor = THUtils.round(zoomPct / 100, 2)
        local zoomTitle = string.format("%d%%", zoomPct)
        settingConstructionMaxZoom:addValue(zoomFactor, zoomTitle)
    end
    settingConstructionMaxZoom:setValueByIndex(1, true, true)
    local settingConstructionDisableCollision = settingsManager:addSetting("constructionCameraDisableCollision", categoryMain, "$l10n_thTitle_constructionCameraDisableCollision", "$l10n_thToolTip_constructionCameraDisableCollision", THSettingsManager.ELEMENT_TYPE.CHECKED)
    settingConstructionDisableCollision:setValue(false, true, true)
end
local function runScript()
    local thMain = THMain.new()
    if thMain ~= nil then
        local modPath = thMain.modPath
        local xmlDataKey = thMain.xmlDataKey
        g_xmlManager:addCreateSchemaFunction(function()
            THMain.xmlSchema = XMLSchema.new(xmlDataKey)
            THMain.xmlSchemaSettings = XMLSchema.new(xmlDataKey .. "_modSettings")
            THMain.xmlSchemaSavegame = XMLSchema.new(xmlDataKey .. "_savegame")
        end)
        g_xmlManager:addInitSchemaFunction(function()
            THMain.initXMLSchema(THMain.xmlSchema, xmlDataKey)
            THMain.initXMLSchemaSettings(THMain.xmlSchemaSettings, xmlDataKey)
            THMain.initXMLSchemaSavegame(THMain.xmlSchemaSavegame, xmlDataKey)
        end)
        _G.g_thMain = thMain
        source(modPath .. "scripts/__core/objects/THNetworkUpdater.lua")
        source(modPath .. "scripts/__core/events/THDebugEvents.lua")
        source(modPath .. "scripts/__core/misc/THAbstractSystem.lua")
        source(modPath .. "scripts/__core/misc/THSettingsManager.lua")
        source(modPath .. "scripts/gui/THConstructionScreenCamera.lua")
    end
end
runScript()