-- Copyright 2024 by Todd Hundersmarck (ThundR)
-- All Rights Reserved


THSetDebugFlagEvent = {}
local THSetDebugFlagEvent_mt = THUtils.createClass(THSetDebugFlagEvent, Event)
InitEventClass(THSetDebugFlagEvent, "THSetDebugFlagEvent")
function THSetDebugFlagEvent.emptyNew()
    return Event.new(THSetDebugFlagEvent_mt)
end
function THSetDebugFlagEvent.new(flagId, isEnabled)
    local self = THSetDebugFlagEvent.emptyNew()
    self.flagId = flagId
    self.isEnabled = isEnabled
    return self
end
function THSetDebugFlagEvent:readStream(streamId, connection)
    if not connection:getIsServer() then -- server reading from client
        if streamReadBool(streamId) then -- flagId exists
            self.flagId = streamReadString(streamId)
        end
        self.isEnabled = streamReadBool(streamId)
        self:run(connection)
    end
end
function THSetDebugFlagEvent:writeStream(streamId, connection)
    if connection:getIsServer() then -- client sending to server
        if streamWriteBool(streamId, self.flagId ~= nil) then
            streamWriteString(streamId, self.flagId)
        end
        streamWriteBool(streamId, self.isEnabled)
    end
end
function THSetDebugFlagEvent:run(connection)
    if not connection:getIsServer() then
        THUtils.setDebugFlag(self.flagId, self.isEnabled)
    end
end
function THSetDebugFlagEvent.sendEvent(flagId, isEnabled)
    THUtils.call(function()
        local mission = g_thMain.mission
        if THUtils.argIsValid(type(isEnabled) == "boolean", "isEnabled", isEnabled, true) then -- ! stop on error
            if not g_thMain.isServer and g_client ~= nil and mission ~= nil then
                if mission.isMasterUser then
                    if flagId ~= nil then
                        flagId = tostring(flagId)
                    end
                    local newEvent = THSetDebugFlagEvent.new(flagId, isEnabled)
                    if newEvent ~= nil then
                        g_client:getServerConnection():sendEvent(newEvent)
                    else
                        THUtils.errorMsg(true, "Could not create THSetDebugFlagEvent") -- ! stop on error
                    end
                else
                    THUtils.displayMsg("Console command requires admin rights to send to server")
                end
            end
        end
    end)
end
THSetDebugLevelEvent = {}
local THSetDebugLevelEvent_mt = THUtils.createClass(THSetDebugLevelEvent, Event)
InitEventClass(THSetDebugLevelEvent, "THSetDebugLevelEvent")
function THSetDebugLevelEvent.emptyNew()
    return Event.new(THSetDebugLevelEvent_mt)
end
function THSetDebugLevelEvent.new(debugLevel)
    local self = THSetDebugLevelEvent.emptyNew()
    self.debugLevel = debugLevel
    return self
end
function THSetDebugLevelEvent:readStream(streamId, connection)
    if not connection:getIsServer() then -- server reading from client
        self.debugLevel = streamReadFloat32(streamId)
        self:run(connection)
    end
end
function THSetDebugLevelEvent:writeStream(streamId, connection)
    if connection:getIsServer() then -- client sending to server
        streamWriteFloat32(streamId, self.debugLevel)
    end
end
function THSetDebugLevelEvent:run(connection)
    if not connection:getIsServer() then -- server reading from client
        THUtils.setDebugLevel(self.debugLevel)
    end
end
function THSetDebugLevelEvent.sendEvent(debugLevel)
    THUtils.call(function()
        local mission = g_thMain.mission
        if THUtils.argIsValid(type(debugLevel) == "number", "debugLevel", debugLevel, true) then  -- ! stop on error (internal)
            if not g_thMain.isServer and g_client ~= nil and mission ~= nil then
                if mission.isMasterUser then
                    local newEvent = THSetDebugLevelEvent.new(debugLevel)
                    if newEvent ~= nil then
                        g_client:getServerConnection():sendEvent(newEvent)
                    else
                        THUtils.errorMsg(true, "Could not create THSetDebugLevelEvent") -- ! stop on error
                    end
                else
                    THUtils.displayMsg("Console command requires admin rights to send to server")
                end
            end
        end
    end)
end