-- Copyright 2024 by Todd Hundersmarck (ThundR)
-- All Rights Reserved


THAbstractSystem = {}
local THAbstractSystem_mt = THUtils.createClass(THAbstractSystem)
function THAbstractSystem.initXMLSchema(xmlSchema, xmlPath)
end
function THAbstractSystem.initXMLSchemaSettings(xmlSchema, xmlPath)
end
function THAbstractSystem.initXMLSchemaSavegame(xmlSchema, xmlPath)
end
function THAbstractSystem.new(xmlDataKey, customMt)
    customMt = customMt or THAbstractSystem_mt
    if THUtils.argIsValid(type(xmlDataKey) == "string" and xmlDataKey ~= "", "xmlDataKey", xmlDataKey, true) then -- ! stop on error
        local self = setmetatable({}, customMt)
        self.isServer = g_thMain.isServer
        self.isClient = g_thMain.isClient
        self.isEnabled = true
        self.modName = g_thMain.modName
        self.modPath = g_thMain.modPath
        self.i18n = g_thMain.i18n
        self.messageCenter = g_thMain.messageCenter
        self.inputBinding = g_thMain.inputBinding
        self.xmlDataKey = xmlDataKey
        self.baseUIFilename = g_thMain.baseUIFilename
        self.baseHUDFilename = g_thMain.baseHUDFilename
        self.settingsXMLPath = g_thMain.modSettingsXMLPath
        self.settingsXMLFilename = string.format("%s%s.xml", self.settingsXMLPath, xmlDataKey)
        if xmlDataKey ~= g_thMain.xmlDataKey then
            self.savegameXMLFilename = "/" .. string.format("%s_%s.xml", g_thMain.xmlDataKey, xmlDataKey)
        else
            self.savegameXMLFilename = "/" .. xmlDataKey .. ".xml"
        end
        self.isInitFinished = false
        return self
    end
end
function THAbstractSystem:initializeDataTables()
    return true
end
function THAbstractSystem:finalizeDataTables()
    return true
end
function THAbstractSystem:loadModSettings()
    local xmlSchema = self.xmlSchemaSettings
    local xmlDataKey = self.xmlDataKey
    local loadSuccess = true
    if xmlSchema ~= nil then
        local xmlFile = XMLFile.loadIfExists(xmlDataKey .. "SettingsXML", self.settingsXMLFilename, xmlSchema)
        if xmlFile ~= nil then
            loadSuccess = self:loadModSettingsFromXMLFile(xmlFile, xmlDataKey)
            xmlFile:delete()
        end
    end
    return loadSuccess
end
function THAbstractSystem:loadModSettingsFromXMLFile(xmlFile, xmlKey)
    return true
end
function THAbstractSystem:saveModSettings()
    local xmlSchema = self.xmlSchemaSettings
    local xmlDataKey = self.xmlDataKey
    local saveSuccess = true
    if xmlSchema ~= nil then
        local xmlFile = nil -- placeholder
        if fileExists(self.settingsXMLFilename) then
            xmlFile = XMLFile.load(xmlDataKey .. "SettingsXML", self.settingsXMLFilename, xmlSchema)
        else
            xmlFile = XMLFile.create(xmlDataKey .. "SettingsXML", self.settingsXMLFilename, xmlDataKey, xmlSchema)
        end
        if xmlFile ~= nil then
            saveSuccess = self:saveModSettingsToXMLFile(xmlFile, xmlDataKey)
            if saveSuccess then
                xmlFile:save()
            end
            xmlFile:delete()
        end
    end
    return saveSuccess
end
function THAbstractSystem:saveModSettingsToXMLFile(xmlFile, xmlKey)
    return true
end
function THAbstractSystem:loadMapSettings()
    local xmlSchema = self.xmlSchema
    local xmlDataKey = self.xmlDataKey
    local loadSuccess = true
    if xmlSchema ~= nil then
        local xmlFile = g_thMain:getMapXMLFile(xmlDataKey, xmlSchema)
        if xmlFile ~= nil then
            loadSuccess = self:loadMapSettingsFromXMLFile(xmlFile, xmlDataKey)
            xmlFile:delete()
        end
    end
    return loadSuccess
end
function THAbstractSystem:loadMapSettingsFromXMLFile(xmlFile, xmlKey)
    return true
end
function THAbstractSystem:loadConfiguration()
    local xmlSchema = self.xmlSchema
    local xmlDataKey = self.xmlDataKey
    local loadSuccess = true
    if xmlSchema ~= nil then
        local xmlFile = g_thMain:getMapXMLFile(xmlDataKey, xmlSchema)
        if xmlFile ~= nil then
            loadSuccess = self:loadFromXMLFile(xmlFile, xmlDataKey)
            xmlFile:delete()
        end
    end
    return loadSuccess
end
function THAbstractSystem:loadFromXMLFile(xmlFile, xmlKey)
    return true
end
function THAbstractSystem:loadFromSavegame(savegame)
    if self.isServer then -- safety
        local xmlSchema = self.xmlSchemaSavegame
        local xmlDataKey = self.xmlDataKey
        local savegameDirectory = savegame.savegameDirectory
        if xmlSchema ~= nil and savegameDirectory ~= nil
            and self.savegameXMLFilename ~= nil and self.savegameXMLFilename ~= ""
        then
            local savegameXMLFilename = savegameDirectory .. self.savegameXMLFilename
            local savegameXMLFile = XMLFile.loadIfExists(xmlDataKey .. "SavegameXML", savegameXMLFilename, xmlSchema)
            if savegameXMLFile ~= nil then
                g_thMain.taskManager:addSubtask(function()
                    self:loadFromSavegameXMLFile(savegame, savegameXMLFile, xmlDataKey)
                    savegameXMLFile:delete()
                end)
            end
        end
    end
end
function THAbstractSystem:loadFromSavegameXMLFile(savegame, xmlFile, xmlKey)
    return true
end
function THAbstractSystem:saveToSavegame(savegame)
    if self.isServer then -- safety
        local xmlSchema = self.xmlSchemaSavegame
        local xmlDataKey = self.xmlDataKey
        local savegameDirectory = savegame.savegameDirectory
        if xmlSchema ~= nil and savegameDirectory ~= nil
            and self.savegameXMLFilename ~= nil and self.savegameXMLFilename ~= ""
        then
            local savegameXMLFilename = savegameDirectory .. self.savegameXMLFilename
            local savegameXMLFile = XMLFile.create(xmlDataKey .. "SavegameXML", savegameXMLFilename, xmlDataKey, xmlSchema)
            if savegameXMLFile ~= nil then
                THUtils.call(function()
                    if self:saveToSavegameXMLFile(savegame, savegameXMLFile, xmlDataKey) then
                        savegameXMLFile:save()
                    end
                end)
                savegameXMLFile:delete()
            end
        end
    end
end
function THAbstractSystem:saveToSavegameXMLFile(savegame, xmlFile, xmlKey)
    return true
end
function THAbstractSystem:onInitialize()
    if not self.isInitFinished then
        self.isInitFinished = true
    end
end
function THAbstractSystem:onSetMissionInfo(mission, missionInfo, missionDynamicInfo)
    self.mission = mission
    self.missionInfo = missionInfo
    self.missionDynamicInfo = missionDynamicInfo
end
function THAbstractSystem:onLoadMapFinished(mission)
end
function THAbstractSystem:onLoadEnvironment(environment, mission)
end
function THAbstractSystem:onUpdate(dt)
end
function THAbstractSystem:onUpdateTick(dt)
end
function THAbstractSystem:onReadStream(streamId, connection)
end
function THAbstractSystem:onWriteStream(streamId, connection)
end
function THAbstractSystem:onReadUpdateStream(streamId, timeStamp, connection)
end
function THAbstractSystem:onWriteUpdateStream(streamId, connection, dirtyMask)
end
function THAbstractSystem:getIsMasterUser()
    local mission = self.mission
    if mission ~= nil and mission.isMasterUser == true then
        return true
    end
    return false
end
function THAbstractSystem:raiseActive()
    return g_thMain:raiseActive()
end
function THAbstractSystem:getNextDirtyFlag()
    return g_thMain:getNextDirtyFlag()
end
function THAbstractSystem:raiseDirtyFlags(dirtyFlag)
    return g_thMain:raiseDirtyFlags(dirtyFlag)
end