-- Copyright 2024 by Todd Hundersmarck (ThundR)
-- All Rights Reserved


THNetworkUpdater = {}
local THNetworkUpdater_mt = THUtils.createClass(THNetworkUpdater, Object)
InitObjectClass(THNetworkUpdater, "THNetworkUpdater")
local function getCustomData(target)
    return THUtils.getDataTable(target)
end
function THNetworkUpdater.new(isServer, isClient, customMt, target)
    customMt = customMt or THNetworkUpdater_mt
    local self = Object.new(isServer, isClient, customMt)
    local customData = THNetworkUpdaterData.newData(self, target)
    return self
end
function THNetworkUpdater:update(dt, ...)
    local customData = getCustomData(self)
    local target = customData.target
    THNetworkUpdater:superClass().update(self, dt, ...)
    if target.onUpdate ~= nil then
        THUtils.call(function()
            target:onUpdate(dt)
            if target.raiseEvent ~= nil then
                target:raiseEvent("onUpdate", dt)
            end
        end)
    end
end
function THNetworkUpdater:updateTick(dt, ...)
    local customData = getCustomData(self)
    local target = customData.target
    THNetworkUpdater:superClass().updateTick(self, dt, ...)
    if target.onUpdateTick ~= nil then
        THUtils.call(function()
            target:onUpdateTick(dt)
            if target.raiseEvent ~= nil then
                target:raiseEvent("onUpdateTick", dt)
            end
        end)
    end
end
function THNetworkUpdater:readStream(streamId, connection, ...)
    local customData = getCustomData(self)
    local target = customData.target
    THNetworkUpdater:superClass().readStream(self, streamId, connection, ...)
    target:onReadStream(streamId, connection)
    if target.raiseEvent ~= nil then
        target:raiseEvent("onReadStream", streamId, connection)
    end
end
function THNetworkUpdater:writeStream(streamId, connection, ...)
    local customData = getCustomData(self)
    local target = customData.target
    THNetworkUpdater:superClass().writeStream(self, streamId, connection, ...)
    target:onWriteStream(streamId, connection)
    if target.raiseEvent ~= nil then
        target:raiseEvent("onWriteStream", streamId, connection)
    end
end
function THNetworkUpdater:readUpdateStream(streamId, timeStamp, connection, ...)
    local customData = getCustomData(self)
    local target = customData.target
    THNetworkUpdater:superClass().readUpdateStream(self, streamId, timeStamp, connection, ...)
    target:onReadUpdateStream(streamId, timeStamp, connection)
    if target.raiseEvent ~= nil then
        target:raiseEvent("onReadUpdateStream", streamId, timeStamp, connection)
    end
end
function THNetworkUpdater:writeUpdateStream(streamId, connection, dirtyMask, ...)
    local customData = getCustomData(self)
    local target = customData.target
    THNetworkUpdater:superClass().writeUpdateStream(self, streamId, connection, dirtyMask, ...)
    target:onWriteUpdateStream(streamId, connection, dirtyMask)
    if target.raiseEvent ~= nil then
        target:raiseEvent("onWriteUpdateStream", streamId, connection, dirtyMask)
    end
end
THNetworkUpdaterData = {}
local THNetworkUpdaterData_mt = THUtils.createClass(THNetworkUpdaterData)
function THNetworkUpdaterData.newData(parent, target, customMt)
    customMt = customMt or THNetworkUpdaterData_mt
    local self = THUtils.createDataTable(parent, nil, customMt)
    if not self.isDataInitialized[THNetworkUpdaterData] then
        self.isClient = parent.isClient
        self.isServer = parent.isServer
        self.target = target
        self.isRegisteredInStream = false
        self.isDataInitialized[THNetworkUpdaterData] = true
    end
    return self
end