-- Copyright 2024 by Todd Hundersmarck (ThundR)
-- All Rights Reserved


THConstructionScreenCamera = {}
local THConstructionScreenCamera_mt = THUtils.createClass(THConstructionScreenCamera)
local function getCustomData(self)
    return THUtils.getDataTable(self)
end
function THConstructionScreenCamera.newData(parent, customMt)
    customMt = customMt or THConstructionScreenCamera_mt
    local self = getCustomData(parent)
    if self == nil or not self.isDataInitialized[THConstructionScreenCamera] then
        if self == nil then
            self = THUtils.createDataTable(parent, nil, customMt)
        end
        self.messageCenter = g_thMain.messageCenter
        self.settingsManager = g_thMain.settingsManager
        self.cameraMaxZoomFactor = 1
        self.isCameraCollisionDisabled = false
        local maxZoomFactor, _, maxZoomSetting = self.settingsManager:getSettingValue("constructionCameraMaxZoomFactor")
        local isCollisionDisabled, _, disableCollisionSetting = self.settingsManager:getSettingValue("constructionCameraDisableCollision")
        if maxZoomFactor ~= nil then
            self.cameraMaxZoomFactor = THUtils.getNoNil(maxZoomFactor, 1)
            self.settingsManager:addSettingChangedListener(maxZoomSetting, self)
        end
        if isCollisionDisabled ~= nil then
            self.isCameraCollisionDisabled = THUtils.getNoNil(isCollisionDisabled, false)
            self.settingsManager:addSettingChangedListener(disableCollisionSetting, self)
        end
        self.isDataInitialized[THConstructionScreenCamera] = true
    end
    return self
end
function THConstructionScreenCamera:onSettingChanged(settingInfo, newValue, valueIndex)
    local settingsManager = self.settingsManager
    if settingInfo.index == settingsManager.SETTING.CONSTRUCTION_CAMERA_MAX_ZOOM_FACTOR then
        self.cameraMaxZoomFactor = tonumber(newValue) or 1
    elseif settingInfo.index == settingsManager.SETTING.CONSTRUCTION_CAMERA_DISABLE_COLLISION then
        self.isCameraCollisionDisabled = THUtils.getNoNil(newValue, false)
    end
end
function THConstructionScreenCamera:hook_setBrush(superFunc, ...)
    THUtils.call(function()
        local customData = THConstructionScreenCamera.newData(self)
    end)
    return superFunc(self, ...)
end
function THConstructionScreenCamera:hook_update(superFunc, dt, ...)
    local customData = getCustomData(self)
    local functionCacheId = THUtils.createFunctionCacheId(self, THConstructionScreenCamera.hook_update)
    local globalEnv = THUtils.getGlobalEnvironment()
    local camera = self.camera
    local cursor = self.cursor
    local GuiTopDownCamera = globalEnv.GuiTopDownCamera
    local GuiTopDownCursor = globalEnv.GuiTopDownCursor
    THUtils.call(function()
        if customData ~= nil and camera ~= nil and cursor ~= nil
            and GuiTopDownCamera ~= nil and GuiTopDownCursor ~= nil
        then
            local cameraMaxZoomFactor = customData.cameraMaxZoomFactor
            local isCameraCollisionDisabled = customData.isCameraCollisionDisabled
            local function hook_raycastClosest(pSuperFunc, px, py, pz, pdx, pdy, pdz, pMaxRange, pCollisionMask, ...)
                local function appendFunc2(rHitObject, rHitX, rHitY, rHitZ, ...)
                    if pCollisionMask == CollisionMask.ALL - CollisionMask.TRIGGERS
                        and isCameraCollisionDisabled
                    then
                        rHitObject = nil
                    end
                    return rHitObject, rHitX, rHitY, rHitZ, ...
                end
                return appendFunc2(pSuperFunc(px, py, pz, pdx, pdy, pdz, pMaxRange, pCollisionMask, ...))
            end
            THUtils.setCachedHook(functionCacheId, "RaycastUtil", "raycastClosest", hook_raycastClosest, nil, true)
            local function hook_cameraUpdatePosition(pSuperFunc, pSelf, ...)
                local oldDistanceRangeZ = GuiTopDownCamera.DISTANCE_RANGE_Z
                local isRangeUpdated = false
                if pSelf == camera then
                    GuiTopDownCamera.DISTANCE_RANGE_Z = oldDistanceRangeZ * cameraMaxZoomFactor
                    isRangeUpdated = true
                end
                local function appendFunc2(...)
                    if isRangeUpdated then
                        GuiTopDownCamera.DISTANCE_RANGE_Z = oldDistanceRangeZ
                    end
                    return ...
                end
                return appendFunc2(pSuperFunc(pSelf, ...))
            end
            THUtils.setCachedHook(functionCacheId, camera, "updatePosition", hook_cameraUpdatePosition, nil, true)
            local function hook_cursorUpdateRaycast(pSuperFunc, pSelf, ...)
                local oldRaycastDistance = GuiTopDownCursor.RAYCAST_DISTANCE
                local isDistanceUpdated = false
                if pSelf == cursor then
                    GuiTopDownCursor.RAYCAST_DISTANCE = oldRaycastDistance * cameraMaxZoomFactor
                    isDistanceUpdated = true
                end
                local function appendFunc2(...)
                    if isDistanceUpdated then
                        GuiTopDownCursor.RAYCAST_DISTANCE = oldRaycastDistance
                    end
                    return ...
                end
                return appendFunc2(pSuperFunc(pSelf, ...))
            end
            THUtils.setCachedHook(functionCacheId, cursor, "updateRaycast", hook_cursorUpdateRaycast, nil, true)
        end
    end)
    local function appendFunc(...)
        THUtils.restoreAllFunctions(functionCacheId)
        return ...
    end
    return appendFunc(superFunc(self, dt, ...))
end
local function runScript()
    g_thMain:addDelayedLoadTask(function()
        THUtils.setFunctionHook("ConstructionScreen", "setBrush", THConstructionScreenCamera)
        THUtils.setFunctionHook("ConstructionScreen", "update", THConstructionScreenCamera)
    end)
end
runScript()